# -*- coding: utf-8 -*-
#
# Copyright 2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Qt main loop runner."""

# pylint: disable=E0611,F0401

import sys

# pylint: disable=W0621
import dbus.mainloop.qt
# pylint: enable=W0621

from PyQt4 import QtCore


TIMERS = set()


def timeout_func(interval, callback, *a, **kw):
    """Delay import of dynamic bindings to avoid crashes."""

    # QTimers don't support priorities
    kw.pop("priority", None)

    timer = QtCore.QTimer()
    TIMERS.add(timer)

    def _callback():
        """Call the real callback, with arguments, until it returns False."""
        if timer in TIMERS:
            result = callback(*a, **kw)
            if not result:
                timer.stop()
                TIMERS.remove(timer)
                # Probably overkill
                timer.deleteLater()

    timer.timeout.connect(_callback)
    timer.start(interval)


def shutdown_func(*a, **kw):
    """Delay import of dynamic bindings to avoid crashes."""
    QtCore.QCoreApplication.instance().exit()


def start_setup():
    """Setup the env to run the service."""
    # this has to be created before calling dbus.mainloop.qt.DBusQtMainLoop
    loop = QtCore.QCoreApplication(sys.argv)
    dbus.mainloop.qt.DBusQtMainLoop(set_as_default=True)
    return loop


def run_func(loop):
    """Delay import of dynamic bindings to avoid crashes."""
    loop.exec_()
